#
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#

from pathlib import PosixPath

import os

from netq_lib.common import utils
from netq_lib.common.enums import PlatformType
from netq_lib.common.utils import subprocess_wrapper

SWITCHD_SW_TABLE_PATH = '/cumulus/switchd/run/software-tables'
SWITCHD_SW_TABLE_ROOT = '/cumulus/switchd/run/software-tables/root'


class PortHandler(object):
    def __init__(self, logger, sdk_utils):
        self.platform = utils.get_platform_type()
        self._logger = logger
        self.port_map_id = None
        self.portmap = {}
        self.lidmap = {}
        self.port_snapshot = set()
        self.sdk_utils = sdk_utils


    def get_port_name(self, port_id):
        """
        Get port name
        @param port_id: Port identifier. SDK log_port (lid) for CL or if_index for SONiC.
        @return: port name. (eg. 'swp1' in CL or 'Ethernet1' in SONiC)
        """
        if self.platform != PlatformType.SONIC:
            port_id = '0x%x' % port_id
        else:
            port_id = str(port_id)
        return self.portmap.get(port_id) or port_id


    def get_port_lid(self, port_name):
        return self.lidmap.get(port_name, port_name)


    def get_all_port_lids(self):
        return list(self.lidmap.values())


    def _rebuild_port_maps(self):
        try:
            if self.platform == PlatformType.SONIC:
                return self._get_sonic_port_map()
            else:
                return self._get_cl_port_map()
        except Exception as e:
            self._logger.error('Cannot get ports map: %s', str(e))


    def refresh_port_map(self):
        ports = set(os.listdir('/sys/devices/virtual/net'))
        self._logger.debug('port snpshot: %s', list(self.port_snapshot))
        if ports != self.port_snapshot:
            self.port_snapshot = ports
            self._logger.info('port map changed')
            self._rebuild_port_maps()


    def _get_sonic_port_map(self):
        output = self.sdk_utils.get_logical_ports()
        # As for now, SONiC port names cannot be translated to SDK log_port.
        # Hence setting latency/occupancy thresholds on specific ports are not supported.
        self.lidmap = {lid: lid for lid in output.split(',')}
        self.portmap = {}
        p = PosixPath('/sys/devices/virtual/net')
        for if_index_file in p.glob('*/ifindex'):
            if_index = if_index_file.read_text().strip('\n')
            if_name = if_index_file.parent.name
            self._logger.debug("WJH Adding port mapping {}: {}".format(if_index, if_name))
            self.portmap[if_index] = if_name


    def _get_cl_port_map(self):
        port = ''
        self.portmap = {}
        self.lidmap = {}

        if self.port_map_id is None:
            self._get_switchd_mapping_tables()

        if not os.path.exists(SWITCHD_SW_TABLE_ROOT):
            return

        cmd = 'cat {}/{}'.format(SWITCHD_SW_TABLE_PATH, self.port_map_id)
        output = subprocess_wrapper(cmd.split())
        for l in output.splitlines():
            if 'port-info' in l:
                port = l.split()[1]
            if 'lid' in l:
                lid = l.split()[1]
                self.portmap[lid] = port
                self.lidmap[port] = lid


    def _get_switchd_mapping_tables(self):
        if not os.path.exists(SWITCHD_SW_TABLE_ROOT):
            return

        cmd = 'cat {}'.format(SWITCHD_SW_TABLE_ROOT)
        try:
            output = subprocess_wrapper(cmd.split())
        except Exception:
            raise Exception('Cannot get switchd table root')
    
        for l in output.splitlines():
            if 'Backend Port Info (Linux) Table' in l:
                self.port_map_id = int(l.split()[1])
            if 'Backend Bond Lag Info Table' in l:
                self.lag_map_id = int(l.split()[1])
    
        if self.port_map_id == 0:
            raise Exception('Cannot get port map table')
